/*
 * Copyright (c) 2011, Florian Zeitz <florob@babelmonkeys.de>
 *
 * http://cgit.babelmonkeys.de/cgit.cgi/mpdbot/
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice is present in all copies.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#import <ObjFW/ObjFW.h>
#import <ObjXMPP/ObjXMPP.h>

#import "PEPThread.h"

#define NS_PUBSUB @"http://jabber.org/protocol/pubsub"
#define NS_TUNE @"http://jabber.org/protocol/tune"

@implementation PEPThread: OFThread
- (void)dealloc
{
	[conn release];

	[super dealloc];
}

- (id)main
{
	OFString *mpd_host;
	uint16_t mpd_port;
	OFDictionary *environment = [OFApplication environment];
	OFString *mpd_port_string;
	mpd_host = [environment objectForKey: @"MPD_HOST"];
	if (mpd_host == nil)
		mpd_host = @"localhost";
	mpd_port_string = [environment objectForKey: @"MPD_PORT"];
	if (mpd_port_string && [mpd_port_string decimalValue] <= UINT16_MAX)
		mpd_port = (uint16_t) [mpd_port_string decimalValue];
	else
		mpd_port = 6600;
	conn = [[MPDConnection alloc] initWithHost: mpd_host
					      port: mpd_port];
	[conn connect];
	while (1) {
		OFAutoreleasePool *pool = [[OFAutoreleasePool alloc] init];
		@try {
			OFMutableDictionary *response;
			OFString *answer;
			XMPPIQ *tuneIQ;
			OFXMLElement *pubsub, *publish, *item, *tune;

			tuneIQ = [XMPPIQ IQWithType: @"set"
						 ID: [object generateStanzaID]];
			pubsub = [OFXMLElement elementWithName: @"pubsub"
						     namespace: NS_PUBSUB];
			[tuneIQ addChild: pubsub];
			publish = [OFXMLElement elementWithName: @"publish"
						      namespace: NS_PUBSUB];
			[publish addAttributeWithName: @"node"
					  stringValue: NS_TUNE];
			[pubsub addChild: publish];
			item = [OFXMLElement elementWithName: @"item"
						   namespace: NS_PUBSUB];
			[publish addChild: item];
			tune = [OFXMLElement elementWithName: @"tune"
						   namespace: NS_TUNE];
			[item addChild: tune];

			[conn send: @"command_list_begin"];
			[conn send: @"status"];
			[conn send: @"currentsong"];
			[conn send: @"command_list_end"];
			response = [conn response];

			if ([[response objectForKey: @"state"]
			    isEqual: @"play"]) {
				if ((answer =
				    [response objectForKey: @"Artist"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"artist"
						  namespace: NS_TUNE
						stringValue: answer]];
				if ((answer = [response objectForKey: @"Time"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"length"
						  namespace: NS_TUNE
						stringValue: answer]];
				if ((answer =
				    [response objectForKey: @"Album"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"source"
						  namespace: NS_TUNE
						stringValue: answer]];
				if ((answer =
				    [response objectForKey: @"Title"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"title"
						  namespace: NS_TUNE
						stringValue: answer]];
				else if ((answer =
				    [response objectForKey: @"file"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"title"
						  namespace: NS_TUNE
						stringValue: answer]];
				if ((answer =
				    [response objectForKey: @"Track"]))
					[tune addChild: [OFXMLElement
					    elementWithName: @"track"
						  namespace: NS_TUNE
						stringValue: answer]];
			}
			[object sendStanza: tuneIQ];
			[conn idle];
		} @catch (id e) {
			[conn connect];
			[e release];
		}
		[pool release];
	}

	return nil;
}
@end
